## ========================================================= ##
##    __  ___  ___ __  _  _ ___  __ _  _ ___ __    __ ____   ## 
##   (  )(  ,)(  ,)  ,) )( )  ,)/  \ \/ )  ,\  )  /  \_  _)  ##
##   /__\ ) ,\ ) ,\)  \\\// ) ,\ () )  ( ) _/)(__( () ))(    ##
##  (_)(_)___/ ___/_)\_)__)(___/\__/_/\_)_) (____)\__/(__)   ##
##                                                           ##
##  Abbreviated Box Plot                                     ##
##  Written by Kristin Potter                                ##
##  http://www.sci.utah.edu/~kpotter/software/sumplots/      ##
##  Copywrite 2010  kpotter@sci.utah.edu                     ##
## ========================================================= ##
 
## ---------------------------------------------------- ##
## Compute and plot the abbreviated box plot
## ---------------------------------------------------- ##
abbrvboxplot <- function(data, newpage=TRUE, width=NULL,
                        linewidth =1.0, color=rgb(0.15, 0.15, 0.15),
                        show=TRUE, dims =NULL,...)
  {
    ## convert the data to a matrix
    data <- as.matrix(data)
    
    ## set up the look of the plot
    abpgp <- gpar(col=color, lwd=linewidth, linejoin="round")
    
    ## Create the object
    abp <- structure(list(data=data, dims=dims, newpage=newpage,
                          look=gpar(abpgp)),class="abbrvboxplot")
    abp <- compute.abbrvboxplot(abp, width)

    ## If true, plot the object
    if(show)
      abp <- plot(abp, compute=FALSE)

    ## silently return
    invisible(abp)
  }

## ---------------------------------------------------- ##
## Compute the Abbreviated Box Plot
## ---------------------------------------------------- ##
compute.abbrvboxplot <- function(x, width=NULL)
  {  
    ## Get the dimensions of the plot
    if(!is.null(width) || is.null(x$dims) )
      x$dims <- findPlotDimensions(x$data, width)

    ## Save off the dimensions
    dims <- x$dims
        
    ## get the dimensions of the plot
    width <- dims$width
    height <- dims$height
    
    ## widht & height shorthands
    spW <- (width/2)*.5
    spH <- (height/2)*.15 
   
    ## get the size of the spaces in the plot
    sp <- spW*.2
      
    ## go through all of the data sets
    points <- matrix(nrow=4*ncol(x$data), ncol=14)
    for(i in 1:ncol(x$data) )
      {
        ## find the quartile range of the data
        q <- quantile(x$data[,i], names=FALSE, na.rm=TRUE)

        ## the minimum line
        minX <- c(i,       i)
        minY <- c(q[1], q[2])
   
        ## lower quartile
        lXs <- c(     i-sp, i-spW*.75,      i+sp, i+spW*.75)
        lXe <- c(i-spW*.75, i-spW*.75, i+spW*.75, i+spW*.75)
        lYs <- c(   q[2],        q[2],    q[2],        q[2])
        lYe <- c(   q[2], q[2]+spH,    q[2], q[2]+spH)
           
        ## median
        mXs <- c(i-(sp*2), i+(sp*2), i-spW, i+spW)
        mXe <- c(i-spW, i+spW, i-spW, i+spW)
        mYs <- c(q[3], q[3], q[3]-spH, q[3]-spH)
        mYe <- c(q[3], q[3], q[3]+spH, q[3]+spH)
    
        ## upper quartile
        uXs <- c(     i-sp, i-spW*.75,      i+sp, i+spW*.75)
        uXe <- c(i-spW*.75, i-spW*.75, i+spW*.75, i+spW*.75)
        uYs <- c(q[4],        q[4], q[4],        q[4])
        uYe <- c(q[4], q[4]-spH, q[4], q[4]-spH)
   
        ## maximum
        maxX <- c(i,       i)
        maxY <- c(q[4], q[5])

        ## add the line segments for the plot
        index <- (i-1)*4
        points[index+1,] <- c(minX[1], lXs, mXs, uXs, maxX[1])
        points[index+2,] <- c(minX[2], lXe, mXe, uXe, maxX[2])
        points[index+3,] <- c(minY[1], lYs, mYs, uYs, maxY[1])
        points[index+4,] <- c(minY[2], lYe, mYe, uYe, maxY[2])
      }
    
    ## set the object points
    x$points <- points
    
    ## silently return
    invisible(x)
  }

## ---------------------------------------------------- ##
## Plot the abbreviated boxplot
## ---------------------------------------------------- ##
plot.abbrvboxplot <- function(x, compute=TRUE,...)
  {
    ## optionally recompute the plot (default TRUE)
    if(compute)
      x <- compute.abbrvboxplot(x)
    
    ## create a new grid, if requested
    if(x$newpage)
      setupView(x$dims$xRange, x$dims$yRange)
    
    ## draw the plot
    for(i in 1:ncol(x$data) )
      {
        index <- (i-1)*4
        xS <- x$points[index+1,]
        xE <- x$points[index+2,]
        yS <- x$points[index+3,]
        yE <- x$points[index+4,]
        grid.segments(xS, yS, xE, yE,
                      default.units="native", gp=x$look)
      }
    
    ## break down the grid, if requested
    if(x$newpage)
      breakdownView()

    ## silently return
    invisible(x)
  }
