## ==================================================== ##
##     ___ __ ___ ____ ___  __ ___ ___ __    __ ____    ##
##    (   \  ) __)_  _)  ,)(  )  ,)  ,\  )  /  \_  _)   ##
##     ) ) )(\__ \ )(  )  \ )( ) ,\) _/)(__( () ))(     ##
##    (___/__)___/(__)(_)\_)__)___/_) (____)\__/(__)    ##
##                                                      ##
## Distrbution Plot                                     ##
## Written by Kristin Potter                            ##
## http://www.sci.utah.edu/~kpotter/software/sumplots/  ##
## Copywrite 2010  kpotter@sci.utah.edu                 ##
## ==================================================== ##

## ---------------------------------------------------- ##
## Compute and plot the distribution fitting plot
## ---------------------------------------------------- ##
distribplot <- function(data, densfun = "normal", start=NULL,
                        breaks="Sturges", newpage=TRUE, width=NULL,
                        show=TRUE, dims=NULL,...)
  {
    ## Convert the data to a matrix
    data <- as.matrix(data)

    ## Create the object
    dfp <- structure(list(data=data, densfun=densfun,start=start,
                          breaks=breaks, dims=dims, newpage=newpage),
                    class="distribplot")
    dfp <- compute.distribplot(dfp, width)
        
    ## if true, plot the plot
    if(show)
      dfp <- plot(dfp, compute=FALSE)

    ## silently return
    invisible(dfp)
  }

# ---------------------------------------------------- ##
## Compute the distribbution fitting plot
## ---------------------------------------------------- ##
compute.distribplot <- function(x, width=NULL,...)
  {
    ## Get the dimensions of the plot
    if(!is.null(width) || is.null(x$dims) )
      x$dims <- findPlotDimensions(x$data, width)
    dims <- x$dims  
    
    ## get the dimensions of the plot
    width <- dims$width
    height <- dims$height
   
    ## vars to save off data
    allpoints <- vector("list", ncol(x$data))
    
    ## go through all of the x$data sets
    for(i in 1:ncol(x$data))
      {
        ## get the size of the x$dataset
        N <- length(x$data[,i])
 
        ## fit the distribution
        fitr <- fitdistr(x$data[,i], x$densfun, x$start)

        ## get the minimum and maximum across all of the data
        minD <- min(x$data[,i])
        maxD <- max(x$data[,i])
   
        ## calculate the number of pts to plot and the interval
        cts = 10
        interval = (maxD-minD)/10
        x$radius <- interval/20
               
        ## width shorthand
        spW <- (width/2)*.65

        ## calculate the geometry of the display
        points <- matrix(nrow=((cts+1)*2), ncol=2)
        for(l in 1:cts+1)
          {
            pt = minD + (l-1)*interval
            dense <- getPt(pt, fitr, x$densfun)

            points[l, 1] <- c(i-spW*dense)
            points[l, 2] <- c(pt)
            
            points[l+cts, 1] <- c(i+spW*dense)
            points[l+cts, 2] <- c(pt)
            
          }       
        ## save the points
        allpoints[[i]] <- points
      }
    
    ## set the plot data
    x$points <- allpoints    
    
    ## silently return
    invisible(x)
  }

## ---------------------------------------------------- ##
## Get the point from the fit distribution
## ---------------------------------------------------- ##
getPt <- function(inPt, fitr, densfun)
  {
    ## Generate the fit distribution
    if(densfun == "beta")
      pt <- dbeta(inPt, shape1=fitr$estimate[1], shape2=fitr$estimate[2])
    else if(densfun =="cauchy")
      pt <- dcauchy(inPt, location=fitr$estimate[1], scale=fitr$estimate[2])
    else if(densfun == "chi-squared")
      pt <- dchisq(inPt, df=fitr$estimate[1])
    else if(densfun =="exponential")
      pt <- dexp(inPt, rate=fitr$estimate[1])
    else if(densfun =="f")
      pt <- df(inPt, df1=fitr$estimate[1], df2=fitr$estimate[2])
    else if(densfun =="gamma")
      pt <- dgamma(inPt, shape=fitr$estimate[1], rate=fitr$estimate[2])
    else if(densfun =="geometric")
      pt <- dgeom(inPt, prob=fitr$estimate[1])
    else if(densfun =="log-normal" || densfun =="lognormal")
      pt <- dlnorm(inPt)
    else if(densfun =="logistic")
      pt <- dlogis(inPt)
    else if(densfun == "negative binomial")
      pt <- rnegbin(inPt, mu=fitr$estimate[1])
    else if(densfun =="normal")
      pt <- dnorm(inPt, mean=fitr$estimate[1], sd=fitr$estimate[2])
    else if(densfun =="Poisson")
      pt <- dpois(inPt, lambda=fitr$estimate[1])
    else if(densfun == "t")
      pt <- dt(inPt, df=fitr$estimate[1], ncp=fitr$estimate[2])
    else if(densfun ==  "weibull")
      pt <- dweibull(inPt, shape=fitr$estimate[1])
    
    invisible(pt)
  }

## ---------------------------------------------------- ##
## Plot the distribution fitting plot
## ---------------------------------------------------- ##
plot.distribplot <- function(x, compute=FALSE,...)
  {
    ## optionally recompute the plot (default TRUE)
    if(compute)
      x <- compute.distribplot(x)
    
    ## create a new grid, if requested
    if(x$newpage)
      setupView(x$dims$xRange, x$dims$yRange)

    
    ## draw the plot
    for(i in 1:ncol(x$data) )
      {
        grid.circle(x$points[[i]][,1], x$points[[i]][,2], r = x$radius,
                    default.units = "native")  
      }
    
    ## break down the grid, if requested
    if(x$newpage)
      breakdownView()
    
    ## silently return
    invisible(x)    
  }
