## ==================================================== ##
##   __  __ __ __  __ ___ _  _ ____ ___ __    __ ____   ##
##  (  \/  )  \  \/  )  _) \( )_  _)  ,\  )  /  \_  _)  ## 
##   )    ( () )    ( ) _))  (  )(  ) _/)(__( () ))(    ## 
##  (_/\/\_)__/_/\/\_)___)_)\_)(__)(_) (____)\__/(__)   ##
##                                                      ##
##  Moment Plot                                         ##
##  Written by Kristin Potter                           ##
##  http://www.sci.utah.edu/~kpotter/software/sumplots/ ##
##  Copywrite 2010  kpotter@sci.utah.edu                ##
## ==================================================== ##

## ---------------------------------------------------- ##
## Compute and plot the moment plot
## ---------------------------------------------------- ##
momentplot <- function(data, newpage=TRUE, width=NULL,
                       show=TRUE, dims=NULL,...)
  {
    ## convert the data to a matrix
    data = as.matrix(data)
    
    ## create the object
    mp <- structure(list(data=data, dims=dims,
                         newpage=newpage),
                    class="momentplot")

    ## compute the moment plot
    mp=compute.momentplot(mp, width)

    ## if true, plot the plot
    if(show)
      mp = plot(mp, compute=FALSE)

    ## silently return
    invisible(mp)
  }

## ---------------------------------------------------- ##
## Compute the moment plot
## ---------------------------------------------------- ##
compute.momentplot <- function(x, width=NULL)
  {
    ## Get the dimensions of the plot
    if(!is.null(width) || is.null(x$dims) )
      x$dims = findPlotDimensions(x$data, width)
    dims=x$dims  
    
    ## get the dimensions of the plot
    width=dims$width*.25
    height=dims$height*.25
    
    ## find the minimum and maximum data values
    minA = min(x$data)
    maxA = max(x$data)

    ## Save off the drawing data
    allgps=list()
    allsegs=c()
    points=matrix(nrow=4*ncol(x$data), ncol=20)
    polys=matrix(nrow=ncol(x$data), ncol=12) 

    ## go through all of the data sets
    for(i in 1:ncol(x$data) )
    {
       ## find the various statistics
       mean = mean(x$data[,i], na.rm=TRUE)
       std = sd(x$data[,i], na.rm=TRUE)
       skew = skewness(x$data[,i], na.rm=TRUE)
       exKurt = kurtosis(x$data[,i], na.rm = TRUE)-3
       tailing = moment(x$data[,i], order = 5, central=TRUE, na.rm=TRUE)/std
     
       ## - - THE mean glyph - -
       meanPtsXs = c(i-width/2, i)
       meanPtsXe = c(i+width/2, i)
       meanPtsYs= c(mean, mean+height/2)
       meanPtsYe= c(mean, mean-height/2)
       allgps[[1]]=gpar(col=rgb(.8, .15, .15), lwd=2.0, linejoin="round")
       allsegs[1] = 2

       ## - - first std glyph - -
       stdXs = c(i-width/2, i-width/4, i+width/4, i-width/2, i-width/4, i+width/4)
       stdXe = c(i-width/4, i+width/4, i+width/2, i-width/4, i+width/4, i+width/2)
       stdYs = c(mean+std-height/7,  mean+std, mean+std, mean-std+height/7,  mean-std,  mean-std) 
       stdYe = c(mean+std, mean+std, mean+std-height/7,  mean-std,  mean-std,mean-std+height/7)
       allgps[[2]]=gpar(col=rgb(0.0, 0, 1.0), lwd=2.0, linejoin="round")
       allsegs[2] = 8

       ## - - second std glyph - -
       std2Xs = c(i-width/4, i-width/8, i+width/8,i-width/4, i-width/8, i+width/8)
       std2Xe = c(i-width/8, i+width/8, i+width/4,i-width/8, i+width/8, i+width/4)
       std2Ys = c(mean+std*2-height/8, mean+std*2, mean+std*2,mean-std*2+height/8, mean-std*2, mean-std*2)
       std2Ye = c(mean+std*2, mean+std*2, mean+std*2-height/8,mean-std*2, mean-std*2, mean-std*2+height/8)
       allgps[[3]]=gpar(col=rgb(0.11, .57, 1.0), lwd=2.0, linejoin="round")
       allsegs[3] = 14

       ## - - kurtosis glyph - -
       pos1 = mean + (height*.125)+exKurt
       pos2 = mean - (height*.125)-exKurt
       xscale1 = width*.25
       xscale2 = xscale1*.5
       yscale = (height*.125)
       exKurt=-.5
       
       ## peaked distribution
       if(exKurt > 0.25)
         {
           ## clamp the excess kurtosis
           if(exKurt > 3)
             exKurt = 3
           
           pos1 = pos1 +1/exKurt
           pos2 = pos2 -1/exKurt
           xscale1 = xscale1*exKurt
           if(xscale1 > width*.25)
             xscale1 = width*.25
           xscale2 = xscale1*.125*(1/exKurt)
           yscale = -1*(height*.125)*(exKurt)
         }
       ## flat distribution
       else if(exKurt < -0.25)
         {
           ## clamp the excess kurtosis
           if(exKurt < -3)
             exKurt = -3
           
           pos1 = pos1 -exKurt
           pos2 = pos2 +exKurt
           
           yscale = 0 
         }
       kXs = c(i-xscale1, i-xscale2, i+xscale2, i-xscale1, i-xscale2, i+xscale2)
       kXe = c(i-xscale2, i+xscale2, i+xscale1, i-xscale2, i+xscale2, i+xscale1)
       kYs = c(pos1, pos1+yscale, pos1+yscale, pos2, pos2-yscale, pos2-yscale)
       kYe = c(pos1+yscale, pos1+yscale, pos1, pos2-yscale, pos2-yscale, pos2)
       allgps[[4]]=gpar(col=rgb(0.33, .10, 0.55), lwd=2.0, linejoin="round")
       allsegs[4]= 20
              
       ## add the line segments to the object
       index = (i-1)*4
       points[index+1,] = c(meanPtsXs, stdXs, std2Xs, kXs)
       points[index+2,] = c(meanPtsXe, stdXe, std2Xe, kXe)
       points[index+3,] = c(meanPtsYs, stdYs, std2Ys, kYs)
       points[index+4,] = c(meanPtsYe, stdYe, std2Ye, kYe)

       ## - - skew - -
       pos = c(quantile(x$data[,i], .2, na.rm=TRUE))
       if(skew < 0)
         pos = quantile(x$data[,i], .8, na.rm=TRUE)
        scale = abs(skew)*width
        if(scale > 1.5)
          scale = 1
       yPos = pos+scale*height
        if(skew < 0)
          yPos = pos-scale*height    
       skX = c(i-(scale*width/2), i+(scale*width/2), i)
       skY = c(pos, pos, yPos)
       allgps[[5]]=gpar(col=rgb(0.76, .45, 0.0), fill=rgb(1.0, .59, 0.0), lwd=2.0, linejoin="round")

       ## - - tailing - -
       pos = c(quantile(x$data[,i], .8, na.rm=TRUE))
       if(tailing < 0)
          pos = quantile(x$data[,i], .2, na.rm=TRUE)
       scale = abs(tailing)/(width/6)
       if(scale > 1.25)
         scale = 1
       yPos = pos+scale*height
       if(tailing < 0)
         yPos = pos-scale*height
       tX = c(i-(scale*width/6), i+(scale*width/6), i)
       tY = c(pos, pos, yPos)
       allgps[[6]]=gpar(fill=rgb(0.95, .9, 0.0), col=rgb(.90, .85, 0.0), lwd=2.0, linejoin="round")

       # add the polygons into the data structure
       polys[i,] = c(skX, skY, tX, tY)       
     }
    
    ## set the data
    x$gp = allgps
    x$points = points
    x$segs = allsegs
    x$polys = polys   
    
    ## silently return
    invisible(x)

  }

## ---------------------------------------------------- ##
## Plot the moment plot
## ---------------------------------------------------- ##
plot.momentplot <- function(x, compute=TRUE,...)
  {
    ## optionally recompute the plot (default TRUE)
    if(compute)
      x=compute.momentplot(x)

    ## create a new grid, if requested
    if(x$newpage)
      setupView(x$dims$xRange, x$dims$yRange)
    
    ## draw the plot
    gpCount = 1
    for(i in 1:ncol(x$data) )
      {
        index = (i-1)*4

        ## draw the line segments for the moment plot
        start=1
        end=x$segs[1]               
        for(j in 1:length(x$segs))
          {
            xS = x$points[index+1,start:end]
            xE = x$points[index+2,start:end]
            yS = x$points[index+3,start:end]
            yE = x$points[index+4,start:end]
            grid.segments(xS, yS, xE, yE, default.units="native", gp=x$gp[[j]])

            start=end+1
            end=x$segs[j+1]
          }

        ## draw the polygons for the moment plot
        for(p in 1:nrow(x$polys))
          {
            skewX = x$polys[p,1:3]
            skewY = x$polys[p,4:6]
            grid.polygon(skewX, skewY, default.units="native", gp=x$gp[[5]])
            tX = x$polys[p,7:9]
            tY = x$polys[p,10:12]
            grid.polygon(tX, tY, default.units="native", gp=x$gp[[6]])
          }
      }
    
    ## break down the grid, if requested
    if(x$newpage)
      breakdownView()

    ## silently return
    invisible(x)    
  }
