## ==================================================== ##
##     ___ _  _ __  __ ___ ___ _  _ ___ __    __ ____   ##
##    / __) \/ )  \/  )   \  _) \( )  ,\  )  /  \_  _)  ##
##    \__ \\  / )    ( ) ) ) _))  ( ) _/)(__( () ))(    ##
##    (___/__/ (_/\/\_)___/___)_)\_)_) (____)\__/(__)   ##
##                                                      ##
## Symmetric Density Plot                               ##
## Written by Kristin Potter                            ##
## http://www.sci.utah.edu/~kpotter/software/sumplots/  ##
## Copywrite 2010  kpotter@sci.utah.edu                 ##
## ==================================================== ##

## ---------------------------------------------------- ##
## Compute and plot the symmetric density plot
## ---------------------------------------------------- ##
symdenplot <- function(data, newpage=TRUE, width=NULL,
                       outlinecolor="gray", linewidth=1.0,
                       breaks="Sturges", colormap=NULL,
                       show=TRUE, dims= NULL,...)
  {
    ## convert the data to a matrix
    data = as.matrix(data)
    
    ## set up the look of the plots
    allgps=list()
    for(i in 1:length(data) )
      allgps[[i]]=gpar(col=outlinecolor,lwd=linewidth)

    ## create the object
    sdp <- structure(list(data=data, dims=dims,
                          newpage=newpage, look=allgps,
                          breaks=breaks, colormap=colormap),
                     class="symdenplot")

    ## computer the plot
    sdp=compute.symdenplot(sdp, width)
    
    ## if true, plot the object
    if(show)
      sdp=plot.symdenplot(sdp)

    ## silently return
    invisible(sdp)
  }

## ---------------------------------------------------- ##
## Compute the density plot
## ---------------------------------------------------- ##
compute.symdenplot <- function(x,  width=NULL)
  {
    ## get the dimensions of the plot
    if(!is.null(width) || is.null(x$dims))
      x$dims = findPlotDimensions(x$data, x$width)

     ## save off the dimensions
    dims=x$dims
       
    ## get the dimensions of the plot
    width=dims$width
    height=dims$height

    ## create a sequential hcl colormap
    ncolors = 500;
    if(is.null(x$colormap))
      colormapColors = rev(sequential_hcl(ncolors, h = 180, c = 20, l=c(70, 95), power = 3.5))
    else
      {
        colormapColors = colors
        ncolors = length(colormapColors)
      }
 
    ## vars to save off data
    allcolors=list()
    allpoints=vector("list", ncol(x$data))
     
    ## go through all of the x$data sets
    for(i in 1:ncol(x$data))
      {         
        ## get the size of the x$dataset
        N = length(x$data[,i])
        
        ## calculate the histogram of the data  --- OPTIONS(hist/kernel dense)
        density = hist(x=x$data[,i], breaks=x$breaks, plot=FALSE)
        
        ## Get the number of cells and the breaks from the denity estimator
        cts = density$counts
        numcells <- length(cts)
        bks = density$mids
       
        ## calculate the density (probability 0-1) and the polygons
        mydensity = density$density
 
        ## get the minimum and maximum across all of the data
        minD = min(mydensity)
        maxD = max(mydensity)

        ## width shorthand
        spW = (width/2)*.65

        ## calculate the geometry of the display
        points = matrix(nrow=(length(cts)*4), ncol=3)
        for(l in 1:length(cts))
          {
            dense = (mydensity[l]-minD)/(maxD-minD)
            start = (l-1)*4+1
            end = start + 3
            points[start:end,1] = c(i-spW*dense, i+spW*dense, i+spW*dense, i-spW*dense)
            points[start:end,2] = c(bks[l], bks[l], bks[l+1], bks[l+1])
            points[start:end,3] = l;
          }

        ## find the colors for each polygon
        colors = c(colormapColors[1])
        for(n in 1:length(mydensity))
          {
            index = (ncolors-1)*(mydensity[n] - minD)/(maxD-minD)+1
            if(index > ncolors-1) index = ncolors-1
            colors[n] = colormapColors[index]
          }

        ## save the points, colors and gp's
        allpoints[[i]] = points
        allcolors[[i]] = colors
        x$look[[i]]$fill = c(1:numcells)
      }

    ## set the plot data
    x$points = allpoints
    x$colors = allcolors
    
    ## silently return
    invisible(x)
  }

## ---------------------------------------------------- ##
## Plot the symmetric density plot
## ---------------------------------------------------- ##
plot.symdenplot <- function(x, dims, compute=TRUE,...)
  {
    ## optionally recompute the plot (default TRUE)
    if(compute)
      x=compute.symdenplot(x)

    ## create a new grid, if requested
    if(x$newpage)
      setupView(x$dims$xRange, x$dims$yRange)

    ## draw the plot
    for(i in 1:ncol(x$data) )
      {
        palette(x$colors[[i]])
        grid.polygon(x$points[[i]][,1], x$points[[i]][,2],
                     id=x$points[[i]][,3], default.units = "native", gp=x$look[[i]])  
      }

    ## break down the grid, if requested
    if(x$newpage)
      breakdownView()

    ## silently return
    invisible(x)
  }
